@@ -1,5 +1,5 @@
 /* infinote - Collaborative notetaking application
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,35 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/**
+ * SECTION:inf-text-gtk-view
+ * @title: InfTextGtkView
+ * @short_description: Drawing remote cursors and selections in a #GtkTextView
+ * @include: libinftextgtk/inf-text-gtk-view.h
+ * @see_also: #InfTextGtkBuffer, #InfTextGtkViewport
+ * @stability: Unstable
+ *
+ * #InfTextGtkView is a helper object which, as long as it is alive, draws
+ * the cursor position, selected text of remote users into a #GtkTextView
+ * widget. It can also highlight the current line of a remote user in a
+ * similar way the #GtkSourceView widget can highlight the current line of
+ * the local user.
+ *
+ * This functionality was not implemented by subclassing #GtkTextView such
+ * that it can also be used with existing subclasses, such as #GtkSourceView.
+ *
+ * With the function inf_text_gtk_view_set_active_user() the local
+ * #InfTextUser who is editing the text can be set, if there is any. The
+ * insertion position, seleceted text or current line are not drawn for this
+ * user, since the normal #GtkTextView takes care of that already. This
+ * assumes the local user cursor position and selection is synchronized to the
+ * corresponding #InfTextUser properties, which is automatically the case when
+ * a #InfTextGtkBuffer is used for the buffer the #GtkTextView is displaying.
+ *
+ * See #InfTextGtkViewport for drawing a marker at remote users' location into
+ * the scrollbar.
+ */	
+
 #include <libinftextgtk/inf-text-gtk-view.h>
 #include <libinfinity/inf-signals.h>
 #include <gdk/gdk.h>
@@ -398,7 +427,8 @@ inf_text_gtk_view_user_invalidate_user_area(InfTextGtkViewUser* view_user)
 #endif
   {
     /* Invalidate cursors/selections */
-    if(priv->show_remote_cursors || priv->show_remote_selections)
+    if(priv->show_remote_cursors || priv->show_remote_selections ||
+       priv->show_remote_current_lines)
     {
       window = gtk_text_view_get_window(priv->textview, GTK_TEXT_WINDOW_TEXT);
 #if GTK_CHECK_VERSION(2, 91, 0)
@@ -469,26 +499,26 @@ inf_text_gtk_view_user_invalidate_user_area(InfTextGtkViewUser* view_user)
             inf_text_gtk_view_get_left_margin(priv->textview) +
             inf_text_gtk_view_get_right_margin(priv->textview);
         }
+
+        gdk_window_invalidate_rect(window, &invalidate_rect, FALSE);
       }
 
-     gdk_window_invalidate_rect(window, &invalidate_rect, FALSE);
-   }
+      /* Invalidate current lines */
+      if(priv->show_remote_current_lines)
+      {
+        gtk_text_view_buffer_to_window_coords(
+          priv->textview,
+          GTK_TEXT_WINDOW_TEXT,
+          0, view_user->line_y,
+          NULL, &invalidate_rect.y
+        );
 
-    /* Invalidate current lines */
-    if(priv->show_remote_current_lines)
-    {
-      gtk_text_view_buffer_to_window_coords(
-        priv->textview,
-        GTK_TEXT_WINDOW_TEXT,
-        0, view_user->line_y,
-        NULL, &invalidate_rect.y
-      );
-
-      /* -1 to stay consistent with GtkSourceView */
-      invalidate_rect.x =
-        inf_text_gtk_view_get_left_margin(priv->textview) - 1;
-      invalidate_rect.width = window_width - invalidate_rect.x;
-      invalidate_rect.height = view_user->line_height;
+        /* -1 to stay consistent with GtkSourceView */
+        invalidate_rect.x =
+          inf_text_gtk_view_get_left_margin(priv->textview) - 1;
+        invalidate_rect.width = window_width - invalidate_rect.x;
+        invalidate_rect.height = view_user->line_height;
+      }
 
       gdk_window_invalidate_rect(window, &invalidate_rect, FALSE);
     }
@@ -1457,10 +1487,12 @@ inf_text_gtk_view_user_selection_changed_cb_idle_func(gpointer user_data)
   InfTextGtkViewUser* view_user;
   view_user = (InfTextGtkViewUser*)user_data;
 
+  g_assert(view_user->revalidate_idle != 0);
+  view_user->revalidate_idle = 0;
+
   /* Revalidate */
   inf_text_gtk_view_user_invalidate_user_area(view_user);
 
-  view_user->revalidate_idle = 0;
   return FALSE;
 }
 
@@ -1507,12 +1539,15 @@ inf_text_gtk_view_user_selection_changed_cb(InfTextUser* user,
      * coordinates at this point. We need to wait for the textview to
      * revalidate onscreen lines first (which it does in an idle handler,
      * note higher numbers indicate less priority). */
-    view_user->revalidate_idle = g_idle_add_full(
-      GTK_TEXT_VIEW_PRIORITY_VALIDATE + 1,
-      inf_text_gtk_view_user_selection_changed_cb_idle_func,
-      view_user,
-      NULL
-    );
+    if(view_user->revalidate_idle == 0)
+    {
+      view_user->revalidate_idle = g_idle_add_full(
+        GTK_TEXT_VIEW_PRIORITY_VALIDATE + 1,
+        inf_text_gtk_view_user_selection_changed_cb_idle_func,
+        view_user,
+        NULL
+      );
+    }
   }
 }
 
@@ -1904,8 +1939,8 @@ inf_text_gtk_view_dispose(GObject* object)
   view = INF_TEXT_GTK_VIEW(object);
   priv = INF_TEXT_GTK_VIEW_PRIVATE(view);
 
-  inf_text_gtk_view_set_view(view, NULL);
   inf_text_gtk_view_set_user_table(view, NULL);
+  inf_text_gtk_view_set_view(view, NULL);
 
   g_assert(priv->active_user == NULL);
   g_assert(priv->users == NULL);
@@ -2215,7 +2250,7 @@ inf_text_gtk_view_get_text_view(InfTextGtkView* view)
  * Returns the #InfUserTable containing the users of the session the
  * #InfTextGtkView's #GtkTextView is displaying.
  *
- * Returns: The #InfGtkTextView's #InfUserTable.
+ * Returns: The #InfTextGtkView's #InfUserTable.
  */
 InfUserTable*
 inf_text_gtk_view_get_user_table(InfTextGtkView* view)
@@ -2323,7 +2358,7 @@ inf_text_gtk_view_set_show_remote_cursors(InfTextGtkView* view,
  * that user's color on top of the author color which indicates who wrote the
  * selected text. If more than one user has a given piece of text selected
  * then an alternating stripe pattern with each of the user's colors is drawn.
- * If @show is %FALES then selection ranges of remote users are not drawn.
+ * If @show is %FALSE then selection ranges of remote users are not drawn.
  */
 void
 inf_text_gtk_view_set_show_remote_selections(InfTextGtkView* view,
@@ -2351,7 +2386,7 @@ inf_text_gtk_view_set_show_remote_selections(InfTextGtkView* view,
  * If @show is %TRUE then all lines in which the cursor of a non-local user
  * in %INF_USER_ACTIVE status is is highlighted with that user's color, similar
  * to GtkSourceView's "highlight current line" functionality. If it is %FALSE
- * then the current line of non-local users is not be highlighted.
+ * then the current line of non-local users is not highlighted.
  */
 void
 inf_text_gtk_view_set_show_remote_current_lines(InfTextGtkView* view,
